# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
from collections import OrderedDict
import copy
import json
from unittest import mock
from unittest.mock import call

# 3rd party libs
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
from oneview_redfish_toolkit.blueprints import chassis
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestChassis(BaseFlaskTest):
    """Tests for Chassis blueprint

        @Todo(ff) List performed tests
        Tests:
            - enclosures
            - blades
            - racks
                - against know value
                - rack not found
                - unexpected exception
    """

    @classmethod
    def setUpClass(self):
        super(TestChassis, self).setUpClass()

        self.app.register_blueprint(chassis.chassis)

        # Loading Enclosure mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/Enclosure.json'
        ) as f:
            self.enclosure = json.load(f)

        # Loading EnclosureEnvironmentalConfig mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'EnclosureEnvironmentalConfig.json'
        ) as f:
            self.enclosure_environment_configuration_mockup = json.load(f)

        # Loading EnclosureChassis mockup result
        with open(
                'oneview_redfish_toolkit/mockups/redfish/EnclosureChassis.json'
        ) as f:
            self.enclosure_chassis_mockup = json.load(f)

        # Loading server_hardware mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

        # Loading BladeChassis mockup result
        with open(
                'oneview_redfish_toolkit/mockups/redfish/BladeChassis.json'
        ) as f:
            self.blade_chassis_mockup = json.load(f)

        # Loading Rack mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/Rack.json'
        ) as f:
            self.rack = json.load(f)

        # Loading RackChassis mockup result
        with open(
                'oneview_redfish_toolkit/mockups/redfish/RackChassis.json'
        ) as f:
            self.rack_chassis_mockup = json.load(f)

        # Loading ApplianceNodeInfo mockup result
        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ApplianceNodeInfo.json'
        ) as f:
            self.appliance_info = json.load(f)

        self.map_appliance = OrderedDict({
            "10.0.0.1": self.appliance_info["uuid"]
        })

    #############
    # Enclosure #
    #############

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_enclosure_chassis(self, get_map_appliances,
                                   get_map_resources):
        """"Tests EnclosureChassis with a known Enclosure"""

        get_map_resources.return_value = OrderedDict({
            "0000000000A66101": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "enclosures"}]
        self.oneview_client.enclosures.get.return_value = self.enclosure
        self.oneview_client.enclosures.get_environmental_configuration.\
            return_value = self.enclosure_environment_configuration_mockup
        self.oneview_client.appliance_node_information.get_version.return_value = \
            self.appliance_info

        # Get EnclosureChassis
        response = self.client.get(
            "/redfish/v1/Chassis/0000000000A66101"
        )

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.enclosure_chassis_mockup, result)
        self.assertEqual(
            "{}{}".format("W/", self.enclosure["eTag"]),
            response.headers["ETag"])

    @mock.patch.object(multiple_oneview, 'config')
    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_enclosure_not_found(self,
                                     get_map_appliances,
                                     get_map_resources,
                                     config_mock):
        """Tests EnclosureChassis with Enclosure not found"""

        config_mock.get_oneview_multiple_ips.return_value = ['10.0.0.1']

        get_map_resources.return_value = OrderedDict({
            "0000000000A66101": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "enclosures"}]
        e = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'enclosure not found',
        })

        self.oneview_client.enclosures.get.side_effect = e

        response = self.client.get(
            "/redfish/v1/Chassis/0000000000A66101"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch.object(multiple_oneview, 'config')
    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_enclosure_env_config_not_found(self, get_map_appliances,
                                                get_map_resources,
                                                config_mock):
        """Tests EnclosureChassis with Enclosure env_config not found"""

        config_mock.get_oneview_multiple_ips.return_value = ['10.0.0.1']

        get_map_resources.return_value = OrderedDict({
            "0000000000A66101": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "enclosures"}]

        self.oneview_client.enclosures.get.return_value = self.enclosure

        e = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'environmental configuration not found',
        })
        self.oneview_client.enclosures.get_environmental_configuration.\
            side_effect = e

        response = self.client.get(
            "/redfish/v1/Chassis/0000000000A66101"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_enclosure_unexpected_error(self):
        """Tests EnclosureChassis with an unexpected error"""

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "enclosures"}]
        self.oneview_client.enclosures.get.side_effect = Exception()

        response = self.client.get(
            "/redfish/v1/Chassis/0000000000A66101"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_enclosure_env_config_unexpected_error(self):
        """Tests EnclosureChassis env_config with an unexpected error"""

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "enclosures"}]

        self.oneview_client.enclosures.get.return_value = self.enclosure
        self.oneview_client.enclosures.get_environmental_configuration.\
            side_effect = Exception()

        response = self.client.get(
            "/redfish/v1/Chassis/0000000000A66101"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_enclosure_chassis_cached(self, get_map_appliances,
                                          get_map_resources):
        """"Tests EnclosureChassis with a known Enclosure"""

        get_map_resources.return_value = OrderedDict({
            "0000000000A66101": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "enclosures"}]
        self.oneview_client.enclosures.get.return_value = self.enclosure
        self.oneview_client.enclosures.get_environmental_configuration.\
            return_value = self.enclosure_environment_configuration_mockup
        self.oneview_client.appliance_node_information.get_version.return_value = \
            self.appliance_info

        uri = "/redfish/v1/Chassis/0000000000A66101"
        uuid = uri.split('/')[-1]

        # Get EnclosureChassis
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)

        # Get cached EnclosureChassis
        response = self.client.get(uri)

        # Tests cached response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.enclosure_chassis_mockup, result)
        self.assertEqual(
            "{}{}".format("W/", self.enclosure["eTag"]),
            response.headers["ETag"])

        # Check for cached calls
        self.oneview_client.index_resources.get_all.assert_called_once_with(
            filter='uuid=' + uuid
        )
        assert self.oneview_client.enclosures.get.has_calls(
            [call(uuid),
             call(uuid)]
        )
        self.assertTrue(category_resource.get_category_by_resource_id(uuid))

    #############
    # Blade     #
    #############

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_blade_chassis_without_computer_system(self,
                                                       get_map_appliances,
                                                       get_map_resources):
        """"Tests BladeChassis with a known Server Hardware"""

        server_hardware = copy.deepcopy(self.server_hardware)
        server_hardware["serverProfileUri"] = None

        get_map_resources.return_value = OrderedDict({
            "30303437-3034-4D32-3230-313133364752": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.return_value = server_hardware
        self.oneview_client.appliance_node_information.get_version.return_value = \
            self.appliance_info

        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
        )

        result = json.loads(response.data.decode("utf-8"))

        expected_blade_result = copy.deepcopy(self.blade_chassis_mockup)
        expected_blade_result["Links"]["ComputerSystems"] = []

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_blade_result, result)
        self.assertEqual(
            "{}{}".format("W/", server_hardware["eTag"]),
            response.headers["ETag"])
        self.oneview_client.server_hardware.get \
            .assert_called_with(server_hardware["uuid"])

        server_hardware["serverProfileUri"] = ""
        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
        )
        result = json.loads(response.data.decode("utf-8"))

        self.assertEqualMockup(expected_blade_result, result)

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_blade_chassis_with_computer_system(self,
                                                    get_map_appliances,
                                                    get_map_resources):
        """"Tests BladeChassis with a known Server Hardware"""

        get_map_resources.return_value = OrderedDict({
            "30303437-3034-4D32-3230-313133364752": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware
        self.oneview_client.appliance_node_information.get_version.return_value = \
            self.appliance_info

        # Get BladeChassis
        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
        )

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.blade_chassis_mockup, result)
        self.assertEqual(
            "{}{}".format("W/", self.server_hardware["eTag"]),
            response.headers["ETag"])

    @mock.patch.object(multiple_oneview, 'config')
    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_server_hardware_not_found(self, get_map_appliances,
                                           get_map_resources,
                                           config_mock):
        """Tests BladeChassis with Server Hardware not found"""

        config_mock.get_oneview_multiple_ips.return_value = ['10.0.0.1']

        self.oneview_client.index_resources.get_all.return_value = [
            {"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.return_value =\
            {'serverHardwareUri': 'invalidUri'}
        e = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'server hardware not found',
        })

        get_map_resources.return_value = OrderedDict({
            "30303437-3034-4D32-3230-313133364752": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance

        self.oneview_client.server_hardware.get.side_effect = e

        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_server_hardware_unexpected_error(self):
        """Tests BladeChassis with an unexpected error"""

        self.oneview_client.index_resources.get_all.return_value = [
            {"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.side_effect = Exception()

        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_blade_chassis_with_computer_system_cached(self,
                                                           get_map_appliances,
                                                           get_map_resources):
        """"Tests BladeChassis with a known Server Hardware"""

        get_map_resources.return_value = OrderedDict({
            "30303437-3034-4D32-3230-313133364752": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware
        self.oneview_client.appliance_node_information.get_version.return_value = \
            self.appliance_info

        uri = "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
        uuid = uri.split('/')[-1]

        # Get BladeChassis
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)

        # Get cached BladeChassis
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.blade_chassis_mockup, result)
        self.assertEqual(
            "{}{}".format("W/", self.server_hardware["eTag"]),
            response.headers["ETag"])

        # Check for cached calls
        self.oneview_client.index_resources.get_all.assert_called_once_with(
            filter='uuid=' + uuid
        )
        assert self.oneview_client.server_hardware.get.has_calls(
            [call(uuid),
             call(uuid)]
        )
        self.assertTrue(category_resource.get_category_by_resource_id(uuid))

    ########
    # Rack #
    ########

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_rack_chassis(self, get_map_appliances, get_map_resources):
        """"Tests RackChassis with a known Rack"""

        get_map_resources.return_value = OrderedDict({
            "2AB100LMNB": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "racks"}]
        self.oneview_client.racks.get.return_value = self.rack

        # Get RackChassis
        response = self.client.get(
            "/redfish/v1/Chassis/2AB100LMNB"
        )

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.rack_chassis_mockup, result)
        self.assertEqual(
            "{}{}".format("W/", self.rack["eTag"]),
            response.headers["ETag"])

    @mock.patch.object(multiple_oneview, 'config')
    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_rack_not_found(self, get_map_appliances, get_map_resources,
                                config_mock):
        """Tests RackChassis with Racks not found"""

        config_mock.get_oneview_multiple_ips.return_value = ['10.0.0.1']

        get_map_resources.return_value = OrderedDict({
            "2AB100LMNB": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "racks"}]
        self.oneview_client.racks.get.return_value = {'rackeUri': 'invalidUri'}
        e = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'rack not found',
        })

        self.oneview_client.racks.get.side_effect = e

        response = self.client.get(
            "/redfish/v1/Chassis/2AB100LMNB"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_rack_unexpected_error(self):
        """Tests RackChassis with an unexpected error"""

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "racks"}]
        self.oneview_client.racks.get.side_effect = Exception()

        response = self.client.get(
            "/redfish/v1/Chassis/2AB100LMNB"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_rack_chassis_cached(self, get_map_appliances,
                                     get_map_resources):
        """"Tests RackChassis with a known Rack"""

        get_map_resources.return_value = OrderedDict({
            "2AB100LMNB": "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "racks"}]
        self.oneview_client.racks.get.return_value = self.rack

        uri = "/redfish/v1/Chassis/2AB100LMNB"
        uuid = uri.split('/')[-1]

        # Get RackChassis
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)

        # Get cached RackChassis
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.rack_chassis_mockup, result)
        self.assertEqual(
            "{}{}".format("W/", self.rack["eTag"]),
            response.headers["ETag"])

        # Check for cached calls
        self.oneview_client.index_resources.get_all.assert_called_once_with(
            filter='uuid=' + uuid
        )
        assert self.oneview_client.racks.get.has_calls(
            [call(uuid),
             call(uuid)]
        )
        self.assertTrue(category_resource.get_category_by_resource_id(uuid))

    def test_change_power_state(self):
        """Tests changes a SH chassi type with valid reset options

            Valid Reset Values:
                - On
                - ForceOff
                - GracefulShutdown
                - GracefulRestart
                - ForceRestart
                - PushPowerButton
        """
        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware
        self.oneview_client.server_hardware.update_power_state.return_value = \
            {"status": "OK"}

        reset_types = ["On", "ForceOff", "GracefulShutdown",
                       "GracefulRestart", "ForceRestart", "PushPowerButton"]

        for reset_type in reset_types:
            response = self.client.post(
                "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
                "/Actions/Chassis.Reset",
                data=json.dumps(dict(ResetType=reset_type)),
                content_type='application/json')

            json_str = response.data.decode("utf-8")

            self.assertEqual(status.HTTP_200_OK, response.status_code)
            self.assertEqual("application/json", response.mimetype)
            self.assertEqual(json_str, '{"ResetType": "%s"}' % reset_type)

    def test_change_power_state_invalid_value(self):
        """Tests changes a SH chassi type with invalid power value"""

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware

        response = self.client.post(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
            "/Actions/Chassis.Reset",
            data=json.dumps(dict(ResetType="INVALID_TYPE")),
            content_type='application/json')

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_change_power_state_unexpected_error(self):
        """Tests changes a SH chassi type with OneView unexpected error"""

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.side_effect = Exception()

        response = self.client.post(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
            "/Actions/Chassis.Reset",
            data=json.dumps(dict(ResetType="On")),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_change_power_state_oneview_exception(self):
        """Tests changes a SH chassi type with OneView unexpected error"""

        e = HPOneViewException({
            'errorCode': 'ANOTHER_ERROR',
            'message': 'server-hardware error',
        })

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.side_effect = e

        response = self.client.post(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
            "/Actions/Chassis.Reset",
            data=json.dumps(dict(ResetType="On")),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_change_power_state_unable_reset(self):
        """Tests changes a SH chassi type with SH unable to reset"""

        e = HPOneViewException({
            'errorCode': 'INVALID_POWER_CONTROL_REQUEST_POWER_COLDBOOT_OFF',
            'message': 'Unable to cold boot because the server is '
                       'currently off.'
        })

        self.oneview_client.index_resources.get_all.return_value = \
            [{"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware
        self.oneview_client.server_hardware.update_power_state.side_effect = e

        response = self.client.post(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
            "/Actions/Chassis.Reset",
            data=json.dumps(dict(ResetType="ForceRestart")),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)

    def test_change_power_state_invalid_key(self):
        """Tests a SH chassi type with JSON key different of ResetType"""

        response = self.client.post(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
            "/Actions/Chassis.Reset",
            data=json.dumps(dict(INVALID_KEY="On")),
            content_type='application/json')
        error_msg = "Invalid JSON key: {}".format("INVALID_KEY")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(
            status.HTTP_400_BAD_REQUEST,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)
        self.assertEqual(error_msg, result["error"]["message"])

    def test_get_sh_with_resource_index_not_found(self):
        """"Tests ServerHardware with empty index resource"""

        self.oneview_client.index_resources.get_all.return_value = \
            []

        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364123"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_get_chassis_throwing_exception(self):
        self.oneview_client.index_resources.get_all.return_value = [
            {"category": "server-hardware"}]
        self.oneview_client.server_hardware.get.side_effect = \
            Exception("connection error")

        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json
from unittest import mock

# 3rd party libs
from flask_api import status

# Module libs
from oneview_redfish_toolkit.blueprints.subscription import subscription
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestSubscription(BaseFlaskTest):
    """Tests for Subscription blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestSubscription, self).setUpClass()

        self.app.register_blueprint(subscription)

    @mock.patch('uuid.uuid1')
    def test_add_subscription(self, uuid_mockup):
        """Test POST Subscription"""

        uuid_mockup.return_value = "e7f93fa2-0cb4-11e8-9060-e839359bc36a"

        response = self.client.post(
            "/redfish/v1/EventService/EventSubscriptions/",
            data=json.dumps(dict(
                Destination="http://www.dnsname.com/Destination1",
                EventTypes=["Alert", "ResourceUpdated"])),
            content_type='application/json')

        result = json.loads(response.data.decode("utf-8"))

        with open(
            'oneview_redfish_toolkit/mockups/'
            'redfish/Subscription.json'
        ) as f:
            subscription_mockup = json.loads(f.read())

        self.assertEqual(status.HTTP_201_CREATED, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(subscription_mockup, result)

    @mock.patch('uuid.uuid1')
    def test_add_subscription_duplicated_event_types(self, uuid_mockup):
        """Test POST Subscription with duplicated event types"""

        uuid_mockup.return_value = "e7f93fa2-0cb4-11e8-9060-e839359bc36a"

        response = self.client.post(
            "/redfish/v1/EventService/EventSubscriptions/",
            data=json.dumps(dict(
                Destination="http://www.dnsname.com/Destination1",
                EventTypes=["Alert", "Alert", "ResourceUpdated"])),
            content_type='application/json')

        result = json.loads(response.data.decode("utf-8"))

        with open(
                'oneview_redfish_toolkit/mockups/'
                'redfish/Subscription.json'
        ) as f:
            subscription_mockup = json.loads(f.read())

        self.assertEqual(status.HTTP_201_CREATED, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(subscription_mockup, result)

    @mock.patch('uuid.uuid1')
    def test_add_subscription_invalid_key1(self, uuid_mockup):
        """Test POST Subscription with invalid Destination key"""

        uuid_mockup.return_value = "e7f93fa2-0cb4-11e8-9060-e839359bc36a"

        response = self.client.post(
            "/redfish/v1/EventService/EventSubscriptions/",
            data=json.dumps(dict(
                INVALID="http://www.dnsname.com/Destination1",
                EventTypes=["Alert", "ResourceUpdated"])),
            content_type='application/json')

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch('uuid.uuid1')
    def test_add_subscription_invalid_key2(self, uuid_mockup):
        """Test POST Subscription with invalid EventTypes key"""

        uuid_mockup.return_value = "e7f93fa2-0cb4-11e8-9060-e839359bc36a"

        response = self.client.post(
            "/redfish/v1/EventService/EventSubscriptions/",
            data=json.dumps(dict(
                Destination="http://www.dnsname.com/Destination1",
                INVALID=["Alert", "ResourceUpdated"])),
            content_type='application/json')

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch('uuid.uuid1')
    def test_add_subscription_empty_events(self, uuid_mockup):
        """Test POST Subscription with empty list of EventTypes"""

        uuid_mockup.return_value = "e7f93fa2-0cb4-11e8-9060-e839359bc36a"

        response = self.client.post(
            "/redfish/v1/EventService/EventSubscriptions/",
            data=json.dumps(dict(
                Destination="http://www.dnsname.com/Destination1",
                EventTypes=[])),
            content_type='application/json')

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch('uuid.uuid1')
    def test_add_subscription_invalid_events(self, uuid_mockup):
        """Test POST Subscription with invalid EventTypes"""

        uuid_mockup.return_value = "e7f93fa2-0cb4-11e8-9060-e839359bc36a"

        response = self.client.post(
            "/redfish/v1/EventService/EventSubscriptions/",
            data=json.dumps(dict(
                Destination="http://www.dnsname.com/Destination1",
                EventTypes=["INVALID"])),
            content_type='application/json')

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch('uuid.uuid1')
    def test_add_subscription_with_context(self, uuid_mockup):
        """Test POST Subscription with a Context"""

        uuid_mockup.return_value = "e7f93fa2-0cb4-11e8-9060-e839359bc36a"

        response = self.client.post(
            "/redfish/v1/EventService/EventSubscriptions/",
            data=json.dumps(dict(
                Destination="http://www.dnsname.com/Destination1",
                EventTypes=["Alert"],
                Context="WebUser3")),
            content_type='application/json')

        self.assertEqual(status.HTTP_201_CREATED, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch('uuid.uuid1')
    def test_add_subscription_invalid_destination(self, uuid_mockup):
        """Test POST Subscription with a invalid Destination URI"""

        uuid_mockup.return_value = "e7f93fa2-0cb4-11e8-9060-e839359bc36a"

        response = self.client.post(
            "/redfish/v1/EventService/EventSubscriptions/",
            data=json.dumps(dict(
                Destination="INVALID",
                EventTypes=["Alert"])),
            content_type='application/json')

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch('uuid.uuid1')
    def test_remove_subscription(self, uuid_mockup):
        """Test REMOVE Subscription"""

        uuid_mockup.return_value = "e7f93fa2-0cb4-11e8-9060-e839359bc36b"

        self.client.post(
            "/redfish/v1/EventService/EventSubscriptions/",
            data=json.dumps(dict(
                Destination="http://www.dnsname.com/Destination1",
                EventTypes=["Alert", "ResourceUpdated"])),
            content_type='application/json')

        delete_response = self.client.delete(
            "/redfish/v1/EventService/EventSubscriptions/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36b")

        self.assertEqual(status.HTTP_200_OK, delete_response.status_code)
        self.assertEqual("application/json", delete_response.mimetype)

        get_response = self.client.get(
            "/redfish/v1/EventService/EventSubscriptions/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36b")

        self.assertEqual(
            status.HTTP_404_NOT_FOUND, get_response.status_code)
        self.assertEqual("application/json", get_response.mimetype)

    def test_remove_subscription_invalid_id(self):
        """Test REMOVE Subscription with invalid ID"""

        response = self.client.delete(
            "/redfish/v1/EventService/EventSubscriptions/INVALID")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch('uuid.uuid1')
    def test_get_subscription(self, uuid_mockup):
        """Test GET Subscription"""

        uuid_mockup.return_value = "e7f93fa2-0cb4-11e8-9060-e839359bc36a"

        self.client.post(
            "/redfish/v1/EventService/EventSubscriptions/",
            data=json.dumps(dict(
                Destination="http://www.dnsname.com/Destination1",
                EventTypes=["Alert", "ResourceUpdated"])),
            content_type='application/json')

        with open(
            'oneview_redfish_toolkit/mockups/'
            'redfish/Subscription.json'
        ) as f:
            subscription_mockup = json.loads(f.read())

        response = self.client.get(
            "/redfish/v1/EventService/EventSubscriptions/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36a")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(subscription_mockup, result)

    def test_get_invalid_subscription(self):
        """Test GET invalid Subscription"""

        response = self.client.get(
            "/redfish/v1/EventService/EventSubscriptions/INVALID")

        self.assertEqual(
            status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.errors import OneViewRedfishError
from oneview_redfish_toolkit.api.network_port import \
    NetworkPort
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestNetworkPort(BaseTest):
    """Tests for NetworkPort class"""

    def setUp(self):
        """Tests preparation"""

        # Loading ServerHardware mockup
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

        # Loading NetworkPort mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'NetworkPort1-Ethernet.json'
        ) as f:
            self.network_port_mockup = json.load(f)

        self.device_id = "3"
        self.port_id = "1"

    def test_class_instantiation(self):
        # Tests if class is correctly instantiated and validated

        try:
            network_port = \
                NetworkPort(
                    self.device_id,
                    self.port_id,
                    self.server_hardware)
        except Exception as e:
            self.fail("Failed to instantiate NetworkPort class."
                      " Error: {}".format(e))
        self.assertIsInstance(
            network_port,
            NetworkPort)

    def test_serialize(self):
        # Tests the serialize function result against known result

        try:
            network_port = \
                NetworkPort(
                    self.device_id,
                    self.port_id,
                    self.server_hardware)
        except Exception as e:
            self.fail("Failed to instantiate NetworkPort class."
                      " Error: {}".format(e))

        try:
            result = json.loads(network_port.serialize())
        except Exception as e:
            self.fail("Failed to serialize. Error: ".format(e))

        self.assertEqualMockup(self.network_port_mockup, result)

    def test_invalid_port_id(self):
        # Tests if class with an invalid port_id

        try:
            network_port = NetworkPort(
                self.device_id,
                "invalid_port_id",
                self.server_hardware)
        except OneViewRedfishError as e:
            self.assertIsInstance(e, OneViewRedfishError)
        except Exception as e:
            self.fail("Failed to instantiate NetworkPort class."
                      " Error: {}".format(e))
        else:
            self.fail("Class instantiated with invalid parameters."
                      " Error: {}".format(network_port))



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.resource_block_computer_system \
    import ResourceBlockComputerSystem
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestResourceBlockComputerSystem(BaseTest):
    """Tests for ResourceBlockComputerSystem class"""

    def setUp(self):
        """Tests preparation"""

        # Loading ResourceBlockComputerSystem mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish'
            '/ResourceBlockComputerSystem.json'
        ) as f:
            self.computer_system_mockup = json.load(f)

        # Loading ServerHardware mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

        self.manager_uuid = "b08eb206-a904-46cf-9172-dcdff2fa9639"

    def test_class_instantiation(self):
        # Tests if class is correctly instantiated and validated
        try:
            computer_system = ResourceBlockComputerSystem(
                self.server_hardware, self.manager_uuid
            )
        except Exception as e:
            self.fail(
                "Failed to instantiate ResourceBlockComputerSystem class."
                " Error: {}".format(e))

        self.assertIsInstance(computer_system, ResourceBlockComputerSystem)

    def test_serialize(self):
        # Tests the serialize function result against known result
        try:
            computer_system = ResourceBlockComputerSystem(
                self.server_hardware, self.manager_uuid
            )
        except Exception as e:
            self.fail(
                "Failed to instantiate ResourceBlockComputerSystem class."
                " Error: {}".format(e))
        try:
            result = json.loads(computer_system.serialize())
        except Exception as e:
            self.fail("Failed to serialize. Error: {}".format(e))

        self.assertEqualMockup(self.computer_system_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.thermal import Thermal
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestThermal(BaseTest):
    """Tests for Thermal class"""

    def setUp(self):
        """Tests preparation"""

        # Loading OneView SH Utilization mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerHardwareUtilization.json'
        ) as f:
            self.server_hardware_utilization = json.load(f)

        # Loading BladeChassisThermal mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/BladeChassisThermal.json'
        ) as f:
            self.blade_thermal_mockup = json.load(f)

        # Loading OneView Enclosure Utilization mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/EnclosureUtilization.json'
        ) as f:
            self.enclosure_utilization = json.load(f)

        # Loading EnclosureChassisThermal mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'EnclosureChassisThermal.json'
        ) as f:
            self.enclosure_thermal_mockup = json.load(f)

        # Loading OneView Rack Topology mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/RackDeviceTopology.json'
        ) as f:
            self.rack_utilization = json.load(f)

        # Loading RackChassisThermal mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/RackChassisThermal.json'
        ) as f:
            self.rack_thermal_mockup = json.load(f)

    def test_class_instantiation_for_blade(self):
        # Tests if class is correctly instantiated and validated

        try:
            obj = Thermal(self.server_hardware_utilization, 'uuid', 'Blade')
        except Exception as e:
            self.fail("Failed to instantiate Thermal class."
                      " Error: {}".format(e))
        self.assertIsInstance(obj, Thermal)

    def test_class_instantiation_for_enclosure(self):
        # Tests if class is correctly instantiated and validated

        try:
            obj = Thermal(self.enclosure_utilization, 'uuid', 'Enclosure')
        except Exception as e:
            self.fail("Failed to instantiate Thermal class."
                      " Error: {}".format(e))
        self.assertIsInstance(obj, Thermal)

    def test_class_instantiation_for_rack(self):
        # Tests if class is correctly instantiated and validated

        try:
            obj = Thermal(self.rack_utilization, 'uuid', 'Rack')
        except Exception as e:
            self.fail("Failed to instantiate Thermal class."
                      " Error: {}".format(e))
        self.assertIsInstance(obj, Thermal)

    def test_serialize_for_blade(self):
        # Tests the serialize function result against known result

        try:
            obj = Thermal(
                self.server_hardware_utilization,
                "36343537-3338-4448-3538-4E5030333434",
                "Blade")
        except Exception as e:
            self.fail("Failed to instantiate Thermal class."
                      " Error: {}".format(e))

        try:
            result = json.loads(obj.serialize())
        except Exception as e:
            self.fail("Failed to serialize. Error: ".format(e))

        self.assertEqualMockup(self.blade_thermal_mockup, result)

    def test_serialize_for_enclosure(self):
        # Tests the serialize function result against known result

        try:
            obj = Thermal(
                self.enclosure_utilization,
                "0000000000A66101",
                "Enclosure")
        except Exception as e:
            self.fail("Failed to instantiate Thermal class."
                      " Error: {}".format(e))

        try:
            result = json.loads(obj.serialize())
        except Exception as e:
            self.fail("Failed to serialize. Error: ".format(e))

        self.assertEqualMockup(self.enclosure_thermal_mockup, result)

    def test_serialize_for_rack(self):
        # Tests the serialize function result against known result

        try:
            obj = Thermal(
                self.rack_utilization,
                "2AB100LMNB",
                "Rack")
        except Exception as e:
            self.fail("Failed to instantiate Thermal class."
                      " Error: {}".format(e))

        try:
            result = json.loads(obj.serialize())
        except Exception as e:
            self.fail("Failed to serialize. Error: ".format(e))

        self.assertEqualMockup(self.rack_thermal_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for event_service.py
"""

import json
from unittest import mock

from oneview_redfish_toolkit.api import event_service
from oneview_redfish_toolkit.api.event_service import EventService
from oneview_redfish_toolkit.tests.base_test import BaseTest


@mock.patch.object(event_service, 'config')
class TestEventService(BaseTest):
    """Tests for EventService class"""

    def test_when_enabled(self, config_mock):
        config_mock.auth_mode_is_conf.return_value = True
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'EventService.json'
        ) as f:
            event_service_mockup = json.load(f)

        manager_collection = EventService(3, 30)
        result = json.loads(manager_collection.serialize())

        self.assertEqualMockup(event_service_mockup, result)

    def test_when_disabled(self, config_mock):
        config_mock.auth_mode_is_conf.return_value = False
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'EventServiceDisabled.json'
        ) as f:
            event_service_mockup = json.load(f)

        manager_collection = EventService(2, 10)
        result = json.loads(manager_collection.serialize())

        self.assertEqualMockup(event_service_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections

from oneview_redfish_toolkit.api.metadata import Metadata
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestOdata(BaseTest):
    """Tests for Metadata class"""

    def setUp(self):
        """Tests preparation"""
        self.schemas = collections.OrderedDict()
        self.schemas["ComputerSystemCollection"] = \
            "ComputerSystemCollection.json"
        self.schemas["ComputerSystem"] = "ComputerSystem.v1_4_0.json"

    def test_class_instantiation(self):
        """Tests class instantiation and validation"""

        try:
            metadata = Metadata(self.schemas)
        except Exception as e:
            self.fail("Failed to instantiate Metadata. Error: ".format(e))
        self.assertIsInstance(metadata, Metadata)

    def test_serialize(self):
        """Tests the serialize function result against known result"""

        metadata = Metadata(self.schemas)
        result = metadata.serialize()

        with open(
            'oneview_redfish_toolkit/mockups/redfish/Metadata.xml'
        ) as f:
            metadata_mockup = f.read()
        self.assertEqualMockup(metadata_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import collections
from unittest import mock

# 3rd party libs
from flask_api import status

# Module libs
from oneview_redfish_toolkit.blueprints.metadata import metadata
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


schemas_dict = collections.OrderedDict()
schemas_dict["ComputerSystemCollection"] = \
    "ComputerSystemCollection.json"
schemas_dict["ComputerSystem"] = "ComputerSystem.v1_4_0.json"


class Metadata(BaseFlaskTest):
    """Tests for Metadata blueprint"""

    @classmethod
    def setUpClass(self):
        super(Metadata, self).setUpClass()

        self.app.register_blueprint(metadata)

    @mock.patch('oneview_redfish_toolkit.api.schemas.SCHEMAS', schemas_dict)
    def test_get_metadata(self):
        """Tests Metadata blueprint result against know value """

        response = self.client.get("/redfish/v1/$metadata")

        result = response.data.decode("utf-8")

        with open(
            'oneview_redfish_toolkit/mockups/redfish/Metadata.xml'
        ) as f:
            metadata_mockup = f.read()

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("text/xml", response.mimetype)
        self.assertEqual(metadata_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import copy
import json
from unittest import mock

# 3rd party libs
from flask_api import status
from hpOneView import HPOneViewException


# Module libs
from oneview_redfish_toolkit.blueprints.session \
    import session as session_blueprint
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit.blueprints.zone_collection import zone_collection
from oneview_redfish_toolkit import client_session
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import connection
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


@mock.patch.object(client_session, 'uuid')
@mock.patch.object(connection, 'OneViewClient')
class TestSession(BaseFlaskTest):
    """Tests for Session blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestSession, self).setUpClass()

        self.app.register_blueprint(session_blueprint)
        self.app.register_blueprint(zone_collection)

        @self.app.errorhandler(status.HTTP_401_UNAUTHORIZED)
        def unauthorized_error(error):
            """Creates a Unauthorized Error response"""
            return ResponseBuilder.error_401(error)

        self.session_id = "e2807c0b-87d6-4304-a773-6ec33521fb1c"
        self.token_id = "abc"

        self.session_ids = [
            'e2807c0b-87d6-4304-a773-6ec33521fb1c',
            '709cf49b-f367-417e-af70-ede782e0fa3c',
            'cd825af3-05c5-44a1-85f7-84e6e2e9fcb3'
        ]

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'SessionCollection.json'
        ) as f:
            self.session_collection = json.load(f)

    def _build_common_sessions(self, uuid_mock):
        uuid_mock.uuid4.side_effect = self.session_ids

        client_session.init_map_clients()
        client_session._set_new_client_by_token('abc', '10.0.0.1')
        client_session._set_new_client_by_token('def', '10.0.0.2')
        client_session._set_new_client_by_token('ghi', '10.0.0.3')

    def test_get_session_collection(self, _, uuid_mock):
        """Tests get Session Collection"""

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'SessionCollection.json'
        ) as f:
            expected_session_collection = json.load(f)

        uuid_mock.uuid4.side_effect = self.session_ids

        client_session.init_map_clients()
        client_session._set_new_client_by_token('abc', '10.0.0.1')
        client_session._set_new_client_by_token('def', '10.0.0.2')
        client_session._set_new_client_by_token('ghi', '10.0.0.3')

        response = self.client.get(
            "/redfish/v1/SessionService/Sessions",
            content_type='application/json')

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_session_collection, result)

    def test_get_session_collection_when_is_empty(self, _, __):
        """Tests get Session Collection when active session list is empty"""
        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'SessionCollectionEmpty.json'
        ) as f:
            expected_session_collection = json.load(f)

        client_session.init_map_clients()

        response = self.client.get(
            "/redfish/v1/SessionService/Sessions",
            content_type='application/json')

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_session_collection, result)

    def test_get_session(self, _, uuid_mock):
        """Tests get a specific Session"""

        with open(
                'oneview_redfish_toolkit/mockups/redfish/Session.json'
        ) as f:
            expected_session = json.load(f)

        uuid_mock.uuid4.side_effect = self.session_ids

        client_session.init_map_clients()
        client_session._set_new_client_by_token('abc', '10.0.0.1')
        client_session._set_new_client_by_token('def', '10.0.0.2')

        response = self.client.get(
            "/redfish/v1/SessionService/Sessions/" + expected_session["Id"],
            content_type='application/json')

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_session, result)

    def test_get_session_when_is_not_cached(self, _, uuid_mock):
        """Tests get a specific Session when session id is not present"""

        uuid_mock.uuid4.return_value = '709cf49b-f367-417e-af70-ede782e0fa3c'

        client_session.init_map_clients()
        client_session._set_new_client_by_token('abc', '10.0.0.1')

        response = self.client.get(
            "/redfish/v1/SessionService/Sessions/"
            "e2807c0b-87d6-4304-a773-6ec33521fb1c",
            content_type='application/json')

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch.object(config, 'get_authentication_mode')
    def test_post_session(self,
                          get_authentication_mode,
                          oneview_client_mockup,
                          uuid_mock):
        """Tests post Session"""

        # Loading Session mockup result
        with open(
                'oneview_redfish_toolkit/mockups/redfish/Session.json'
        ) as f:
            expected_session_mockup = json.load(f)

        client_session.init_map_clients()
        multiple_oneview.init_map_appliances()
        get_authentication_mode.return_value = 'session'

        # Create mock response
        uuid_mock.uuid4.return_value = self.session_id
        oneview_client = oneview_client_mockup()
        oneview_client.connection.get_session_id.return_value = self.token_id

        # POST Session
        response = self.client.post(
            "/redfish/v1/SessionService/Sessions",
            data=json.dumps(dict(
                UserName="administrator",
                Password="password")),
            content_type='application/json')

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_session_mockup, result)
        self.assertIn("/redfish/v1/SessionService/Sessions/" + self.session_id,
                      response.headers["Location"])
        self.assertEqual(self.token_id, response.headers["X-Auth-Token"])
        oneview_client_mockup.assert_called_with(
            {
                'ip': config.get_oneview_multiple_ips()[0],
                'credentials': {
                    'userName': 'administrator',
                    'password': 'password'
                },
                'api_version': config.get_api_version()
            }
        )

        # gets the session by id
        response_of_get = self.client.get(response.headers["Location"],
                                          content_type='application/json')

        result_of_get = json.loads(response_of_get.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response_of_get.status_code)
        self.assertEqual(self.session_id, result_of_get["Id"])

    def test_post_many_sessions(self, oneview_client_mockup, uuid_mock):
        """Tests post many Sessions and gets them"""

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'SessionCollection.json'
        ) as f:
            expected_session_collection = json.load(f)

        client_session.init_map_clients()
        multiple_oneview.init_map_appliances()

        session_ids = self.session_ids
        uuid_mock.uuid4.side_effect = session_ids

        oneview_client = oneview_client_mockup()
        oneview_client.connection.get_session_id.side_effect = [
            'abc',
            'def',
            'ghi'
        ]

        self.client.post(
            "/redfish/v1/SessionService/Sessions",
            data=json.dumps(dict(
                UserName="user_1",
                Password="password")),
            content_type='application/json')

        self.client.post(
            "/redfish/v1/SessionService/Sessions",
            data=json.dumps(dict(
                UserName="user_2",
                Password="password")),
            content_type='application/json')

        self.client.post(
            "/redfish/v1/SessionService/Sessions",
            data=json.dumps(dict(
                UserName="user_3",
                Password="password")),
            content_type='application/json')

        response = self.client.get('/redfish/v1/SessionService/Sessions',
                                   content_type='application/json')
        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual(expected_session_collection, result)

    @mock.patch.object(config, 'get_authentication_mode')
    def test_post_session_with_login_domain_data(self,
                                                 get_authentication_mode,
                                                 oneview_client_mockup,
                                                 uuid_mock):
        """Tests post Session when UserName has login domain information"""

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'SessionForLoginWithDomain.json'
        ) as f:
            expected_session_mockup = json.load(f)

        client_session.init_map_clients()
        multiple_oneview.init_map_appliances()
        get_authentication_mode.return_value = 'session'

        # Create mock response
        uuid_mock.uuid4.return_value = self.session_id
        oneview_client = oneview_client_mockup()
        oneview_client.connection.get_session_id.return_value = self.token_id

        # POST Session
        response = self.client.post(
            "/redfish/v1/SessionService/Sessions",
            data=json.dumps(dict(
                UserName="LOCAL\\administrator",
                Password="password")),
            content_type='application/json')

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_session_mockup, result)
        self.assertIn("/redfish/v1/SessionService/Sessions/" + self.session_id,
                      response.headers["Location"])
        self.assertEqual(self.token_id, response.headers["X-Auth-Token"])
        oneview_client_mockup.assert_called_with(
            {
                'ip': config.get_oneview_multiple_ips()[0],
                'credentials': {
                    'userName': 'administrator',
                    'password': 'password',
                    'authLoginDomain': 'LOCAL'
                },
                'api_version': config.get_api_version()
            }
        )

    def test_post_session_invalid_key(self, oneview_client_mockup, _):
        """Tests post session with an invalid JSON key"""

        # Create mock response
        oneview_client = oneview_client_mockup()
        oneview_client.connection.get_session_id.return_value = self.token_id

        # POST Session
        response = self.client.post(
            "/redfish/v1/SessionService/Sessions",
            data=json.dumps(dict(
                InvalidKey="administrator",
                Password="password")),
            content_type='application/json')

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        with open(
                'oneview_redfish_toolkit/mockups/errors/'
                'InvalidCredentialsJsonKey.json'
        ) as f:
            invalid_json_key = json.load(f)

        self.assertEqual(
            status.HTTP_400_BAD_REQUEST,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)
        self.assertEqual(result, invalid_json_key)

    def test_post_session_oneview_exception(self, oneview_client_mockup, _):
        """Tests post session with HPOneViewException"""

        oneview_client = oneview_client_mockup()

        e = HPOneViewException({
            'errorCode': 'HTTP_401_UNAUTHORIZED',
            'message': 'Invalid user information',
        })

        oneview_client.connection.get_session_id.side_effect = e

        # POST Session
        response = self.client.post(
            "/redfish/v1/SessionService/Sessions",
            data=json.dumps(dict(
                UserName="administrator",
                Password="password")),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_401_UNAUTHORIZED,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)

    def test_post_session_unexpected_error(self, oneview_client_mockup, _):
        """Tests post session with an unexpected error"""

        oneview_client = oneview_client_mockup()

        oneview_client.connection.get_session_id.side_effect = Exception()

        # POST Session
        response = self.client.post(
            "/redfish/v1/SessionService/Sessions",
            data=json.dumps(dict(
                UserName="administrator",
                Password="password")),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)

    @mock.patch.object(config, 'get_authentication_mode')
    def test_remove_token_when_authorization_token_fail(self,
                                                        get_auth_mode,
                                                        _,
                                                        uuid_mock):
        """Tests removing a session when Oneview raises authorization error"""

        err = HPOneViewException({
            'errorCode': 'AUTHORIZATION',
            'message': 'Invalid user information',
        })

        get_auth_mode.return_value = 'session'

        self.oneview_client.server_profile_templates.get_all.side_effect = err

        self._build_common_sessions(uuid_mock)

        # Raises the error
        response = self.client.get('/redfish/v1/CompositionService/'
                                   'ResourceZones',
                                   headers={'X-Auth-Token': 'abc'},
                                   content_type='application/json')

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_401_UNAUTHORIZED, response.status_code)
        self.assertEqual('application/json', response.mimetype)
        self.assertIn('Invalid user information', str(result))

        # when gets the Active Session should have only 2 sessions
        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'SessionCollection.json'
        ) as f:
            expected_session_collection = json.load(f)
            del expected_session_collection["Members"][0]
            expected_session_collection["Members@odata.count"] = 2

        resp_active_sess = self.client.get(
            "/redfish/v1/SessionService/Sessions",
            content_type='application/json')

        result_active_sess = json.loads(resp_active_sess.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, resp_active_sess.status_code)
        self.assertEqual("application/json", resp_active_sess.mimetype)
        self.assertEqualMockup(expected_session_collection, result_active_sess)

    def test_logout(self, _, uuid_mock):
        """Tests logout successfully"""

        self._build_common_sessions(uuid_mock)

        response = self.client.delete('/redfish/v1/SessionService/Sessions/' +
                                      self.session_ids[0],
                                      headers={'X-Auth-Token': 'abc'},
                                      content_type='application/json')

        self.assertEqual(status.HTTP_204_NO_CONTENT, response.status_code)
        self.assertEqual('application/json', response.mimetype)

        # when gets the Active Session should have only 2 sessions
        expected_session_collection = copy.deepcopy(self.session_collection)
        del expected_session_collection["Members"][0]
        expected_session_collection["Members@odata.count"] = 2

        resp_active_sess = self.client.get(
            "/redfish/v1/SessionService/Sessions",
            content_type='application/json')

        result_active_sess = json.loads(resp_active_sess.data.decode("utf-8"))

        self.assertEqualMockup(expected_session_collection, result_active_sess)

    def test_logout_when_session_belongs_to_other_client(self, _, uuid_mock):
        """Tests logout when session belongs to other client, returns 404"""

        self._build_common_sessions(uuid_mock)

        response = self.client.delete('/redfish/v1/SessionService/Sessions/' +
                                      self.session_ids[0],
                                      headers={'X-Auth-Token': 'def'},
                                      content_type='application/json')

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual('application/json', response.mimetype)

        resp_active_sess = self.client.get(
            "/redfish/v1/SessionService/Sessions",
            content_type='application/json')

        result_active_sess = json.loads(resp_active_sess.data.decode("utf-8"))

        # when gets the Active Session should have all sessions
        self.assertEqualMockup(self.session_collection, result_active_sess)

    def test_logout_when_session_is_invalid(self, _, uuid_mock):
        """Tests logout when session is invalid, returns 404"""

        self._build_common_sessions(uuid_mock)

        response = self.client.delete('/redfish/v1/SessionService/Sessions/' +
                                      '12345678',
                                      headers={'X-Auth-Token': 'abc'},
                                      content_type='application/json')

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual('application/json', response.mimetype)

        resp_active_sess = self.client.get(
            "/redfish/v1/SessionService/Sessions",
            content_type='application/json')

        result_active_sess = json.loads(resp_active_sess.data.decode("utf-8"))

        # when gets the Active Session should have all sessions
        self.assertEqualMockup(self.session_collection, result_active_sess)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json

from flask_api import status

from oneview_redfish_toolkit.blueprints import zone_collection
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestZoneCollection(BaseFlaskTest):
    """Tests for ZoneCollection blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestZoneCollection, self).setUpClass()

        self.app.register_blueprint(zone_collection.zone_collection)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerProfileTemplates.json'
        ) as f:
            self.server_profile_template_list = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/ZoneCollection.json'
        ) as f:
            self.zone_collection_mockup = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/LogicalEnclosures.json'
        ) as f:
            self.logical_encl_list = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/oneview'
                '/DriveEnclosureList.json'
        ) as f:
            self.drive_enclosure_list = json.load(f)

    def test_get_zone_collection_when_get_templates_raises_error(self):
        """Tests ZoneCollection when server profile templates raises error"""

        self.oneview_client.server_profile_templates.get_all.side_effect = \
            Exception("An exception has occurred")

        with open(
            'oneview_redfish_toolkit/mockups/errors/Error500.json'
        ) as f:
            error_500 = json.load(f)

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceZones/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqual(error_500, result)

    def test_get_zone_collection(self):
        """Tests ZoneCollection"""

        ov_api = self.oneview_client

        ov_api.logical_enclosures.get_all.return_value = \
            self.logical_encl_list

        ov_api.server_profile_templates.get_all.return_value = \
            self.server_profile_template_list
        ov_api.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceZones/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)

        self.assertEqualMockup(self.zone_collection_mockup, result)

        ov_api.drive_enclosures.get_all.assert_called_with()
        ov_api.logical_enclosures.get_all.assert_called_with()

    def test_get_zone_collection_empty(self):
        """Tests ZoneCollection with an empty list"""

        self.oneview_client.server_profile_templates.get_all.return_value = \
            []

        with open(
            'oneview_redfish_toolkit/mockups/redfish/ZoneCollectionEmpty.json'
        ) as f:
            zone_collection_empty_mockup = json.load(f)

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceZones/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(zone_collection_empty_mockup, result)

    def test_get_zone_collection_without_drive_enclosure_associated(self):
        ov_api = self.oneview_client
        ov_api.logical_enclosures.get_all.return_value = \
            self.logical_encl_list

        zone_collection_mockup = copy.deepcopy(self.zone_collection_mockup)
        zone_collection_mockup["Members@odata.count"] = 1
        del zone_collection_mockup["Members"][:2]

        ov_api.server_profile_templates.get_all.return_value = \
            self.server_profile_template_list
        ov_api.drive_enclosures.get_all.return_value = []

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceZones/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(zone_collection_mockup, result)

        ov_api.drive_enclosures.get_all.assert_called_with()
        ov_api.logical_enclosures.get_all.assert_called_with()

    def test_get_zone_collection_with_drive_enclosure_without_drives(self):
        ov_api = self.oneview_client

        ov_api.logical_enclosures.get_all.return_value = \
            self.logical_encl_list

        zone_collection_mockup = copy.deepcopy(self.zone_collection_mockup)
        zone_collection_mockup["Members@odata.count"] = 1
        del zone_collection_mockup["Members"][:2]
        drive_enclosure_list = copy.deepcopy(self.drive_enclosure_list)
        del drive_enclosure_list[:1]
        drive_enclosure_list[0]["driveBays"] = 0

        ov_api.server_profile_templates.get_all.return_value = \
            self.server_profile_template_list
        ov_api.drive_enclosures.get_all.return_value = \
            drive_enclosure_list

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceZones/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)

        self.assertEqualMockup(zone_collection_mockup, result)

        ov_api.drive_enclosures.get_all.assert_called_with()
        ov_api.logical_enclosures.get_all.assert_called_with()

    def test_get_zone_collection_mixed_mode(self):
        ov_api = self.oneview_client

        spt_list = \
            copy.deepcopy(self.server_profile_template_list)
        spt_list[1]["localStorage"]["controllers"][0]["mode"] = "Mixed"

        ov_api.logical_enclosures.get_all.return_value = \
            self.logical_encl_list

        ov_api.server_profile_templates.get_all.return_value = \
            spt_list
        ov_api.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceZones/")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.zone_collection_mockup, result)

        ov_api.drive_enclosures.get_all.assert_called_with()
        ov_api.logical_enclosures.get_all.assert_called_with()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.redfish_json_validator import \
    RedfishJsonValidator
from oneview_redfish_toolkit.api import status_mapping


class StorageDriveComposedDetails(RedfishJsonValidator):
    """Creates a Drive dict for a storage of a composed system

            Populates self.redfish with data retrieved from
            an OneView's Drive
    """

    SCHEMA_NAME = 'Drive'

    def __init__(self, drive_id, server_profile, logical_jbod):
        """StorageDriveComposedDetails constructor

            Populates self.redfish with the contents of drive from an
            Oneview's sas logical jbod and an Oneview's server profile

            Args:
                drive_id: id of redfish Drive
                server_profile: Oneview's server profile dict
                logical_jbod: Oneview's sas logical jbod dict
        """
        super().__init__(self.SCHEMA_NAME)

        enclosure_id = server_profile["enclosureUri"].split("/")[-1]
        profile_uuid = server_profile["uri"].split("/")[-1]

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = str(drive_id)
        self.redfish["Name"] = logical_jbod["name"]
        self.redfish["Status"] = status_mapping\
            .STATUS_MAP.get(logical_jbod["status"])

        size_in_bytes = float(logical_jbod["maxSizeGB"]) \
            * 1024 * 1024 * 1024
        self.redfish["CapacityBytes"] = int(size_in_bytes)
        self.redfish["Protocol"] = \
            logical_jbod["driveTechnology"]["deviceInterface"]
        self.redfish["MediaType"] = \
            logical_jbod["driveTechnology"]["driveMedia"]
        self.redfish["Links"] = {
            "Chassis": {
                "@odata.id": "/redfish/v1/Chassis/" + enclosure_id
            }
        }
        self.redfish["@odata.context"] = "/redfish/v1/$metadata#Drive.Drive"

        self.redfish["@odata.id"] = "{}/{}/Storage/1/Drives/{}"\
            .format(ComputerSystem.BASE_URI, profile_uuid, drive_id)

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.server_profile_template_resource_block \
    import ServerProfileTemplateResourceBlock
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestServerProfileTemplateResourceBlock(BaseTest):
    """Tests for ServerProfileTemplateResourceBlock class"""

    def test_serialize(self):
        # Tests the serialize function result against known result

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerProfileTemplate.json'
        ) as f:
            server_profile_template = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish'
            '/ServerProfileTemplateResourceBlock.json'
        ) as f:
            expected_result = json.load(f)

        zone_ids = [
            "1f0ca9ef-7f81-45e3-9d64-341b46cf87e0-0000000000A66101",
            "1f0ca9ef-7f81-45e3-9d64-341b46cf87e0-0000000000A66102"
        ]

        resource_block = ServerProfileTemplateResourceBlock(
            '1f0ca9ef-7f81-45e3-9d64-341b46cf87e0',
            server_profile_template,
            zone_ids)

        result = json.loads(resource_block.serialize())

        self.assertEqualMockup(expected_result, result)

    def test_serialize_with_only_one_zone(self):
        # Tests the serialize function result against known result

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerProfileTemplates.json'
        ) as f:
            server_profile_template = json.load(f)[1]

        with open(
                'oneview_redfish_toolkit/mockups/redfish'
                '/SPTResourceBlockWithOnlyOneZone.json'
        ) as f:
            expected_result = json.load(f)

        zone_ids = [
            expected_result["Id"]
        ]

        resource_block = ServerProfileTemplateResourceBlock(
            expected_result["Id"],
            server_profile_template,
            zone_ids)

        result = json.loads(resource_block.serialize())

        self.assertEqualMockup(expected_result, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import uuid

from flask import abort
from flask import Blueprint
from flask import request
from flask import Response
from flask_api import status
from jsonschema.exceptions import ValidationError
import validators

from oneview_redfish_toolkit.api.errors import OneViewRedfishError
from oneview_redfish_toolkit.api.subscription import Subscription
from oneview_redfish_toolkit import util


subscription = Blueprint("subscription", __name__)


@subscription.route(
    "/redfish/v1/EventService/EventSubscriptions/", methods=["POST"])
def add_subscription():
    """Add the Redfish Subscription.

        Add a new subscription when this POST operation is requested.
        The body of the request must have the Destination,
        an array of EventTypes. Context is optional.

        EventTypes:
            - ResourceUpdated
            - ResourceAdded
            - ResourceRemoved
            - Alert

        Returns:
            JSON: JSON with Subscription information.

        Exception:
            OneViewRedfishError: When occur a key mapping error.
            return abort(400)

            Exception: Unexpected error.
            return abort(500)
    """
    try:
        # get Destination, EventTypes and Context from post
        # generate subscription uuid
        # add subscription in subscriptions_by_type
        # add subscription in all_subscriptions

        try:
            body = request.get_json()
            destination = body["Destination"]

            if not validators.url(destination):
                raise OneViewRedfishError(
                    {"errorCode": "INVALID_INFORMATION",
                     "message": "Destination must be an URI."})

            event_types = body["EventTypes"]

            if not event_types:
                raise OneViewRedfishError(
                    {"errorCode": "INVALID_INFORMATION",
                     "message": "EventTypes cannot be empty."})

            context = body.get("Context")
        except KeyError:
            raise OneViewRedfishError(
                {"errorCode": "INVALID_INFORMATION",
                 "message": "Invalid JSON key. The JSON request body"
                            " must have the keys Destination and EventTypes."
                            " The Context is optional."})

        subscription_id = str(uuid.uuid1())

        try:
            # Build Subscription object and validates it
            sc = Subscription(subscription_id, destination,
                              event_types, context)
        except ValidationError:
            raise OneViewRedfishError(
                {"errorCode": "INVALID_INFORMATION",
                 "message": "Invalid EventType. The EventTypes are "
                            "StatusChange, ResourceUpdated, ResourceAdded,"
                            " ResourceRemoved and Alert."})

        for event_type in sc.get_event_types():
            util.get_subscriptions_by_type()[event_type][subscription_id] = sc

        util.get_all_subscriptions()[subscription_id] = sc

        # Build redfish json
        json_str = sc.serialize()

        # Build response and returns
        response = Response(
            response=json_str,
            status=status.HTTP_201_CREATED,
            mimetype="application/json")
        response.headers.add(
            "Location", "/redfish/v1/EventService/EventSubscriptions/"
                        "{}".format(subscription_id))
        return response

    except OneViewRedfishError as e:
        logging.exception(e)
        abort(status.HTTP_400_BAD_REQUEST, e.msg['message'])
    except Exception as e:
        logging.exception(e)
        return abort(status.HTTP_500_INTERNAL_SERVER_ERROR)


@subscription.route(
    "/redfish/v1/EventService/EventSubscriptions/<subscription_id>",
    methods=["DELETE"])
def remove_subscription(subscription_id):
    """Removes a specific Subscription

        Args:
            subscription_id: The Subscription ID.
    """
    try:
        sc = util.get_all_subscriptions()[subscription_id]
        event_types = sc.get_event_types()

        for event in event_types:
            del util.get_subscriptions_by_type()[event][subscription_id]

        del util.get_all_subscriptions()[subscription_id]

        return Response(
            status=status.HTTP_200_OK,
            mimetype="application/json")
    except KeyError as e:
        logging.exception("Subscription not found: " + str(e))
        abort(status.HTTP_404_NOT_FOUND)


@subscription.route(
    "/redfish/v1/EventService/EventSubscriptions/<subscription_id>",
    methods=["GET"])
def get_subscription(subscription_id):
    """Gets a specific Subscription

        Args:
            subscription_id: The Subscription ID.

        Returns:
            Subscription JSON.
    """
    try:
        sc = util.get_all_subscriptions()[subscription_id]

        json_str = sc.serialize()
        return Response(
            response=json_str,
            status=status.HTTP_200_OK,
            mimetype="application/json")
    except KeyError as e:
        logging.exception("Subscription not found: " + str(e))
        abort(status.HTTP_404_NOT_FOUND)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json
import logging
import os
import pika
import ssl
import threading

# 3rd party libs
from hpOneView.exceptions import HPOneViewException
from hpOneView.oneview_client import OneViewClient
from pika.credentials import ExternalCredentials

# Own libs
from oneview_redfish_toolkit.api.errors import NOT_FOUND_ONEVIEW_ERRORS
from oneview_redfish_toolkit.api.event import Event
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import connection
from oneview_redfish_toolkit import util


SCMB_DIR_NAME = "scmb"
ONEVIEW_CA_NAME = "oneview_ca.pem"
SCMB_CERT_NAME = "oneview_scmb.pem"
SCMB_KEY_NAME = "oneview_scmb.key"
SCMB_PORT = 5671
SCMB_SOCKET_TIMEOUT = 5  # seconds
SCMB_RESOURCE_LIST = [
    'alerts',
    'enclosures',
    'racks',
    'server-hardware']
SCMB_EXCHANGE_NAME = 'scmb'


def _scmb_base_dir():
    certs_dir = os.path.dirname(config.get_config()['ssl']['SSLCertFile'])
    return os.path.join(certs_dir, SCMB_DIR_NAME)


def _oneview_ca_path():
    return os.path.join(_scmb_base_dir(), ONEVIEW_CA_NAME)


def _scmb_cert_path():
    return os.path.join(_scmb_base_dir(), SCMB_CERT_NAME)


def _scmb_key_path():
    return os.path.join(_scmb_base_dir(), SCMB_KEY_NAME)


def init_event_service():
    if config.auth_mode_is_conf():
        # Loading scmb connection
        if _has_valid_certificates():
            logging.info('SCMB certs already exist and are valid...')
        else:
            logging.info('SCMB certs not found. '
                         'Checking if already generated in Oneview...')
            get_cert()
            logging.info('Got certs. Testing connection...')
            if not _is_cert_working_with_scmb():
                logging.error('Failed to connect to scmb. Aborting...')
                exit(1)
        scmb_thread = threading.Thread(target=_listen_scmb)
        scmb_thread.daemon = True
        scmb_thread.start()
    else:
        logging.warning("Authentication mode set to session. SCMB events will "
                        "be disabled")


def _has_valid_certificates():
    try:
        return _has_scmb_certificates_path() and _is_cert_working_with_scmb()
    except KeyError as error:
        logging.error("Invalid configuration for ssl cert. "
                      "Verify the [ssl] section in config file")
        raise error


def _has_scmb_certificates_path():
    return os.path.isfile(_oneview_ca_path()) and \
        os.path.isfile(_scmb_cert_path()) and \
        os.path.isfile(_scmb_key_path())


def get_oneview_client():
    # Workaround for #328
    # Create OneView client using API 500 just to retrieve OneView certificates
    ov_config = connection.create_oneview_config(
        # TODO(victorhugorodrigues): Remove after implementation for handling
        # multiple OneViews for events service
        ip=config.get_oneview_multiple_ips()[0],
        credentials=config.get_credentials(),
        api_version=300
    )
    ov_client = OneViewClient(ov_config)
    ov_client.connection.login(config.get_credentials())

    return ov_client


def get_cert():
    # Get CA
    ov_client = get_oneview_client()

    cert = ov_client.certificate_authority.get()

    # Create the dir to save the scmb files
    os.makedirs(name=_scmb_base_dir(), exist_ok=True)

    with open(_oneview_ca_path(), 'w+') as f:
        f.write(cert)

    certs = _get_scmb_certificate_from_ov(ov_client)

    # Save cert
    with open(_scmb_cert_path(), 'w+') as f:
        f.write(certs['base64SSLCertData'])
    # Save key
    with open(_scmb_key_path(), 'w+') as f:
        f.write(certs['base64SSLKeyData'])


def _get_scmb_certificate_from_ov(ov_client):
    cert = None
    try:
        cert = ov_client.certificate_rabbitmq.get_key_pair('default')
        logging.info('SCMB certs retrieved from OneView.')
    except HPOneViewException as e:
        if e.oneview_response["errorCode"] in NOT_FOUND_ONEVIEW_ERRORS:
            logging.info('Generating SCMB cert in Oneview...')
            _generate_certificate_in_oneview(ov_client)
            cert = ov_client.certificate_rabbitmq.get_key_pair('default')
        else:
            logging.exception("Unexpected error")
            raise e
    return cert


def _generate_certificate_in_oneview(ov_client):
    try:
        cert_info = {
            "commonName": "default",
            "type": "RabbitMqClientCertV2"
        }
        ov_client.certificate_rabbitmq.generate(cert_info)
    except HPOneViewException as e:
        # Cert with that commonName already exists. We are going to get it
        if e.oneview_response["errorCode"] == 'RABBITMQ_CLIENTCERT_CONFLICT':
            logging.info('Certs already exists in oneview')
        else:
            # Another error is not expected, we raise.
            logging.exception("Unexpected error")
            raise e


def scmb_connect():
    # TODO(victorhugorodrigues): Remove after implementation for handling
    # multiple OneViews for events service
    scmb_server = config.get_oneview_multiple_ips()[0]

    # Setup our ssl options
    ssl_options = ({'ca_certs': _oneview_ca_path(),
                    'certfile': _scmb_cert_path(),
                    'keyfile': _scmb_key_path(),
                    'cert_reqs': ssl.CERT_REQUIRED,
                    'server_side': False})

    scmb_connection = pika.BlockingConnection(
        pika.ConnectionParameters(
            scmb_server,
            SCMB_PORT,
            credentials=ExternalCredentials(),
            socket_timeout=SCMB_SOCKET_TIMEOUT,
            ssl=True,
            ssl_options=ssl_options))

    return scmb_connection


def _is_cert_working_with_scmb():
    # Create and bind to queue
    EXCHANGE_NAME = 'scmb'
    ROUTE = 'scmb.alerts.#'
    try:
        scmb_conn = scmb_connect()
        channel = scmb_conn.channel()
        queue = channel.queue_declare(auto_delete=True)
        channel.queue_bind(
            queue=queue.method.queue,
            exchange=EXCHANGE_NAME,
            routing_key=ROUTE)
        channel.close()
        scmb_conn.close()
    except Exception:
        logging.exception("Failed to test scmb connection")
        return False
    return True


def consume_message(ch, method, properties, body):
    body = json.loads(body.decode('utf-8'))
    resource = body['resource']

    if (resource['category'] == 'alerts'):
        category = resource['associatedResource']['resourceCategory']
    else:
        category = resource['category']

    if (category in SCMB_RESOURCE_LIST):
        event = Event(body)

        util.dispatch_event(event)
    else:
        logging.debug('SCMB message received for an unmanaged resource')


def _listen_scmb():
    try:
        scmb_conn = scmb_connect()
        ch = scmb_conn.channel()

        queue_name = ch.queue_declare(auto_delete=True)

        for resource in SCMB_RESOURCE_LIST:
            # scmb.<resource>.#
            route = SCMB_EXCHANGE_NAME + '.' + resource + '.#'

            ch.queue_bind(
                queue=queue_name.method.queue,
                exchange=SCMB_EXCHANGE_NAME,
                routing_key=route)

        ch.basic_consume(consume_message, queue=queue_name.method.queue)
        ch.start_consuming()
    except KeyboardInterrupt:
        ch.close()
        scmb_conn.close()
    except Exception:
        logging.exception("Failed to listen to scmb messages")



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
from collections import OrderedDict
import copy
import json

# 3rd party libs
from unittest import mock

from flask_api import status
from hpOneView.exceptions import HPOneViewException
from unittest.mock import call

# Module libs
import oneview_redfish_toolkit.api.status_mapping as status_mapping
from oneview_redfish_toolkit.blueprints import computer_system
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit.services import computer_system_service
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestComputerSystem(BaseFlaskTest):
    """Tests for ComputerSystem blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestComputerSystem, self).setUpClass()

        self.app.register_blueprint(computer_system.computer_system)

        self.internal_error = HPOneViewException({
            'errorCode': 'ANOTHER_ERROR',
            'message': 'some internal error',
        })

        self.not_found_error = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'resource not found',
        })

        self.not_found_server_profile = HPOneViewException({
            'errorCode': 'ProfileNotFoundException',
            'message': 'The requested profile cannot be retrieved',
        })

        # Loading server_profile mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/ServerProfile.json'
        ) as f:
            self.server_profile = json.load(f)

        # Loading server_hardware mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

        # Loading ServerHardwareTypes mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview'
                '/ServerHardwareTypes.json'
        ) as f:
            self.server_hardware_types = json.load(f)

        # Loading ComputerSystem mockup result
        with open(
                'oneview_redfish_toolkit/mockups/redfish/ComputerSystem.json'
        ) as f:
            self.computer_system_mockup = json.load(f)

        # Loading Drives mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/Drives.json'
        ) as f:
            self.drives = json.load(f)

        # Loading LabelForServerProfile mockup result
        with open(
                'oneview_redfish_toolkit/mockups/oneview'
                '/LabelForServerProfile.json'
        ) as f:
            self.label_for_server_profile = json.load(f)

        # Loading ServerProfileTemplate mockup result
        with open(
                'oneview_redfish_toolkit/mockups/oneview'
                '/ServerProfileTemplate.json'
        ) as f:
            self.server_profile_template = json.load(f)

        self.spt_uri = "/rest/server-profile-templates/" \
                       "61c3a463-1355-4c68-a4e3-4f08c322af1b"

        # Loading ApplianceNodeInfo mockup result
        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ApplianceNodeInfo.json'
        ) as f:
            self.appliance_info = json.load(f)

        self.map_appliance = OrderedDict({
            "10.0.0.1": self.appliance_info["uuid"]
        })

    def test_get_computer_system_not_found(self):
        """Tests ComputerSystem with ServerProfileTemplates not found"""

        self.oneview_client.server_profiles.get.side_effect = \
            self.not_found_server_profile
        self.oneview_client.server_profile_templates.get.side_effect = \
            self.not_found_error

        response = self.client.get(
            "/redfish/v1/Systems/0303437-3034-4D32-3230-313133364752"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get \
            .assert_called_with("0303437-3034-4D32-3230-313133364752")
        self.oneview_client.server_profile_templates.get \
            .assert_called_with("0303437-3034-4D32-3230-313133364752")

    def test_get_computer_system_sh_not_found(self):
        """Tests ComputerSystem with Server Hardware not found"""

        self.oneview_client.server_profiles.get.return_value = {
            "category": "server-profiles",
            "serverHardwareUri": "notFoundUri"
        }
        self.oneview_client.server_hardware.get.side_effect = \
            self.not_found_error

        response = self.client.get(
            "/redfish/v1/Systems/0303437-3034-4D32-3230-313133364752"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_hardware.\
            get.assert_called_with('notFoundUri')
        self.oneview_client.server_hardware_types.get.assert_not_called()

    def test_get_computer_system_sht_not_found(self):
        """Tests ComputerSystem with ServerHardwareType not found"""

        self.oneview_client.server_profiles.get.return_value = {
            "category": "server-profiles",
            "serverHardwareUri": "validURI",
            "serverHardwareTypeUri": "notFoundURI"
        }
        self.oneview_client.server_hardware.get.return_value = {}
        self.oneview_client.server_hardware_types.get.side_effect = \
            self.not_found_error

        response = self.client.get(
            "/redfish/v1/Systems/0303437-3034-4D32-3230-313133364752"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_hardware.get.assert_called_with("validURI")
        self.oneview_client.server_hardware_types.get\
            .assert_called_with("notFoundURI")

    def test_get_computer_system_sh_exception(self):
        """Tests ComputerSystem with ServerHardware exception"""

        self.oneview_client.server_profiles.get.return_value = {
            "category": "server-profiles",
            "serverHardwareUri": "someURI"
        }
        self.oneview_client.\
            server_hardware.get.side_effect = self.internal_error

        response = self.client.get(
            "/redfish/v1/Systems/0303437-3034-4D32-3230-313133364752"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)

    def test_get_computer_system_sht_exception(self):
        """Tests ComputerSystem with ServerHardwareTypes exception"""

        self.oneview_client.server_profiles.get.return_value = \
            {
                'serverHardwareTypeUri': 'invalidUri',
                'serverHardwareUri': 'validUri',
                'category': 'server-profiles'
            }
        self.oneview_client.\
            server_hardware.get.return_value = {}  # some object
        self.oneview_client.server_hardware_types.get.side_effect = \
            self.internal_error

        response = self.client.get(
            "/redfish/v1/Systems/0303437-3034-4D32-3230-313133364752"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get \
            .assert_called_with("0303437-3034-4D32-3230-313133364752")

    def test_get_computer_system_spt_exception(self):
        """Tests ComputerSystem with ServerProfileTemplates exception"""

        self.oneview_client.\
            server_profiles.get.side_effect = self.not_found_error
        self.oneview_client.server_profile_templates.get.side_effect = \
            self.internal_error

        response = self.client.get(
            "/redfish/v1/Systems/0303437-3034-4D32-3230-313133364752"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)

    def test_get_computer_system_unexpected_error(self):
        """Tests ComputerSystem with an unexpected error"""

        self.oneview_client.\
            server_profiles.get.side_effect = self.not_found_error
        self.oneview_client.\
            server_profile_templates.get.side_effect = Exception()

        response = self.client.get(
            "/redfish/v1/Systems/0303437-3034-4D32-3230-313133364752"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_computer_system_server_profile(self, get_map_appliances,
                                                get_map_resources):
        """Tests ComputerSystem with a known Server Profile"""

        server_profile = copy.deepcopy(self.server_profile)
        server_profile["localStorage"]["sasLogicalJBODs"].pop(0)

        server_profile_template = copy.deepcopy(self.server_profile_template)
        server_profile_template["uri"] = self.spt_uri

        # Create mock response
        resource_id = "/rest/server-hardware-types/" \
                      "FE50A6FE-B1AC-4E42-8D40-B73CA8CC0CD2"
        get_map_resources.return_value = OrderedDict({
            resource_id: "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.server_profiles.get.return_value = server_profile
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware
        self.oneview_client.server_hardware_types.get.return_value = \
            self.server_hardware_types
        self.oneview_client.sas_logical_jbods.get_drives.return_value = \
            [self.drives[4]]
        self.oneview_client.labels.get_by_resource.return_value = \
            self.label_for_server_profile
        self.oneview_client.server_profile_templates.get.return_value = \
            server_profile_template
        self.oneview_client.appliance_node_information.get_version.return_value = \
            self.appliance_info

        # Get ComputerSystem
        response = self.client.get(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2"
        )

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.computer_system_mockup, result)
        self.assertEqual(
            "{}{}".format("W/", self.server_profile["eTag"]),
            response.headers["ETag"])
        self.oneview_client.server_profiles.get.assert_called_with(
            "b425802b-a6a5-4941-8885-aab68dfa2ee2"
        )
        self.oneview_client.server_hardware.get.assert_called_with(
            "/rest/server-hardware/30303437-3034-4D32-3230-313130304752"
        )
        self.oneview_client.server_hardware_types.get.assert_called_with(
            "/rest/server-hardware-types/FE50A6FE-B1AC-4E42-8D40-B73CA8CC0CD2"
        )
        self.oneview_client.sas_logical_jbods.get_drives.assert_called_with(
            "/rest/sas-logical-jbods/9e83a03d-7a84-4f0d-a8d7-bd05a30c3175"
        )

        self.oneview_client.labels.get_by_resource.assert_called_with(
            "/rest/server-profiles/b425802b-a6a5-4941-8885-aab68dfa2ee2"
        )

        self.oneview_client.server_profile_templates.get.assert_called_with(
            "61c3a463-1355-4c68-a4e3-4f08c322af1b"
        )

    def test_get_computer_system_spt(self):
        """Tests ComputerSystem with a known Server Profile Templates"""

        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerProfileTemplate.json'
        ) as f:
            server_profile_template = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/CapabilitiesObject.json'
        ) as f:
            capabilities_obj_mockup = json.load(f)

        # Create mock response
        self.oneview_client.\
            server_profiles.get.side_effect = self.not_found_error
        self.oneview_client.server_profile_templates.get.return_value = \
            server_profile_template

        # Get ComputerSystem
        response = self.client.get(
            "/redfish/v1/Systems/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
        )

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(capabilities_obj_mockup, result)
        self.oneview_client.server_profiles.get \
            .assert_called_with("1f0ca9ef-7f81-45e3-9d64-341b46cf87e0")
        self.oneview_client.server_profile_templates.get \
            .assert_called_with("1f0ca9ef-7f81-45e3-9d64-341b46cf87e0")

    def test_get_computer_system_spt_cached(self):
        """Tests ComputerSystem with a known Server Profile Templates"""

        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerProfileTemplate.json'
        ) as f:
            server_profile_template = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/CapabilitiesObject.json'
        ) as f:
            capabilities_obj_mockup = json.load(f)

        # Create mock response
        self.oneview_client.\
            server_profiles.get.side_effect = self.not_found_error
        self.oneview_client.server_profile_templates.get.return_value = \
            server_profile_template

        uri = "/redfish/v1/Systems/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
        uuid = uri.split('/')[-1]

        # Get ComputerSystem
        response = self.client.get(uri)

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(capabilities_obj_mockup, result)

        # Get cached ComputerSystem
        response = self.client.get(uri)

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(capabilities_obj_mockup, result)
        self.oneview_client.server_profiles.get \
            .assert_called_once_with(uuid)
        self.oneview_client.server_profile_templates.get \
            .assert_has_calls(
                [call(uuid),
                 call(uuid)]
                )
        self.assertTrue(category_resource.get_category_by_resource_id(uuid))

    def test_change_power_state(self):
        """Tests change SH power state with valid power values

            Valid Power Values:
                - On
                - ForceOff
                - GracefulShutdown
                - GracefulRestart
                - ForceRestart
                - PushPowerButton
        """

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware
        self.oneview_client.server_hardware.update_power_state.return_value = \
            {"status": "OK"}

        reset_types = ["On", "ForceOff", "GracefulShutdown",
                       "GracefulRestart", "ForceRestart", "PushPowerButton"]

        for reset_type in reset_types:
            response = self.client.post(
                "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2"
                "/Actions/ComputerSystem.Reset",
                data=json.dumps(dict(ResetType=reset_type)),
                content_type='application/json')

            # Tests response
            self.assertEqual(status.HTTP_200_OK, response.status_code)
            self.assertEqual("application/json", response.mimetype)

            json_str = response.data.decode("utf-8")

            self.assertEqual(json_str, '{"ResetType": "%s"}' % reset_type)

        self.oneview_client.server_profiles.get.assert_called_with(
            "b425802b-a6a5-4941-8885-aab68dfa2ee2"
        )
        self.oneview_client.server_hardware.get.has_calls(
            [call("/rest/server-hardware/"
                  "30303437-3034-4D32-3230-313130304752"),
             call("30303437-3034-4D32-3230-313130304752")]
        )
        self.oneview_client.server_hardware.update_power_state \
            .assert_called_with({
                'powerControl': 'MomentaryPress',
                'powerState': 'Off'
            }, "30303437-3034-4D32-3230-313133364752")

    def test_change_power_state_invalid_value(self):
        """Tests change SH power state with invalid power value"""

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware

        response = self.client.post(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2"
            "/Actions/ComputerSystem.Reset",
            data=json.dumps(dict(ResetType="INVALID_TYPE")),
            content_type='application/json')

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            "b425802b-a6a5-4941-8885-aab68dfa2ee2"
        )
        self.oneview_client.server_hardware.get.assert_called_with(
            "/rest/server-hardware/30303437-3034-4D32-3230-313130304752"
        )

    def test_change_power_state_unexpected_error(self):
        """Tests change SH power state with OneView unexpected error"""

        self.oneview_client.server_hardware.get.side_effect = Exception()

        response = self.client.post(
            "/redfish/v1/Systems/30303437-3034-4D32-3230-313133364752"
            "/Actions/ComputerSystem.Reset",
            data=json.dumps(dict(ResetType="On")),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_change_power_state_sh_exception(self):
        """Tests change SH power state with SH exception"""

        self.oneview_client.\
            server_hardware.get.side_effect = self.internal_error

        response = self.client.post(
            "/redfish/v1/Systems/30303437-3034-4D32-3230-313133364752"
            "/Actions/ComputerSystem.Reset",
            data=json.dumps(dict(ResetType="On")),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)

    def test_change_power_state_unable_reset(self):
        """Tests change SH power state with SH unable to reset"""

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware
        self.oneview_client.server_hardware.update_power_state.side_effect = \
            self.internal_error

        response = self.client.post(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2"
            "/Actions/ComputerSystem.Reset",
            data=json.dumps(dict(ResetType="ForceRestart")),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            "b425802b-a6a5-4941-8885-aab68dfa2ee2"
        )
        self.oneview_client.server_hardware.get.has_calls(
            [call("/rest/server-hardware/"
                  "30303437-3034-4D32-3230-313130304752"),
             call("/rest/server-hardware/"
                  "30303437-3034-4D32-3230-313130304752")]
        )

    def test_change_power_state_invalid_key(self):
        """Tests change SH power state with JSON key different of ResetType"""

        response = self.client.post(
            "/redfish/v1/Systems/30303437-3034-4D32-3230-313133364752"
            "/Actions/ComputerSystem.Reset",
            data=json.dumps(dict(INVALID_KEY="On")),
            content_type='application/json')

        result = json.loads(response.data.decode("utf-8"))

        with open(
                'oneview_redfish_toolkit/mockups/errors/'
                'InvalidJsonKey.json'
        ) as f:
            invalid_json_key = json.load(f)

        self.assertEqual(
            status.HTTP_400_BAD_REQUEST,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)
        self.assertEqual(result, invalid_json_key)

    def test_remove_computer_system(self):
        """Tests delete computer system"""

        self.oneview_client.server_profiles.get.return_value = {
            'serverHardwareUri': '/rest/server-hardware/uuid_1'
        }
        self.oneview_client.server_profiles.delete.return_value = True

        response = self.client.delete(
            "/redfish/v1/Systems/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36b")

        self.assertEqual(
            status.HTTP_204_NO_CONTENT,
            response.status_code
        )
        self.oneview_client.server_hardware.update_power_state\
            .assert_called_with({
                'powerControl': 'PressAndHold', 'powerState': 'Off'
            }, '/rest/server-hardware/uuid_1')

    @mock.patch.object(computer_system_service, 'config')
    def test_remove_when_power_off_on_decompose_is_not_configured(self,
                                                                  config_mock):
        """Tests delete that should does not power off server"""

        self.oneview_client.server_profiles.get.return_value = {
            'serverHardwareUri': '/rest/server-hardware/uuid_1'
        }
        config_mock.get_composition_settings.return_value = {
            'PowerOffServerOnDecompose': ''
        }
        self.oneview_client.server_profiles.delete.return_value = True

        response = self.client.delete(
            "/redfish/v1/Systems/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36b")

        self.assertEqual(
            status.HTTP_204_NO_CONTENT,
            response.status_code
        )
        self.oneview_client.server_hardware.update_power_state\
            .assert_not_called()

    @mock.patch.object(computer_system_service, 'config')
    def test_remove_when_power_off_on_decompose_has_wrong_configuration(
            self, config_mock):
        """Tests delete that should raise a validation error"""

        self.oneview_client.server_profiles.get.return_value = {
            'serverHardwareUri': '/rest/server-hardware/uuid_1'
        }
        config_mock.get_composition_settings.return_value = {
            'PowerOffServerOnDecompose': 'ForceOffff'
        }

        response = self.client.delete(
            "/redfish/v1/Systems/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36b")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_403_FORBIDDEN, response.status_code)
        self.assertIn('There is no mapping for ForceOffff on the OneView',
                      str(result))

        self.oneview_client.server_hardware.update_power_state\
            .assert_not_called()
        self.oneview_client.server_profiles.delete.assert_not_called()

    def test_remove_computer_system_sp_not_found(self):
        """Tests remove ComputerSystem with ServerProfile not found"""

        self.oneview_client.server_profiles.delete.side_effect = \
            self.not_found_error

        response = self.client.delete(
            "/redfish/v1/Systems/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36b")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_remove_computer_system_exception(self):
        """Tests remove ComputerSystem with ServerProfile exception"""

        e = HPOneViewException({
            'errorCode': 'ANOTHER_ERROR',
            'message': 'server-profile error',
        })
        self.oneview_client.server_profiles.delete.side_effect = e

        response = self.client.delete(
            "/redfish/v1/Systems/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36b")

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_remove_computer_system_not_deleted(self):
        """Tests remove ComputerSystem with ServerProfile not deleted"""

        self.oneview_client.server_profiles.delete.return_value = False

        response = self.client.delete(
            "/redfish/v1/Systems/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36b")

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_remove_computer_system_task_completed(self):
        """Tests remove ComputerSystem with task completed"""

        task = {'taskState': 'Completed'}

        self.oneview_client.server_profiles.delete.return_value = task

        response = self.client.delete(
            "/redfish/v1/Systems/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36b")

        self.assertEqual(
            status.HTTP_204_NO_CONTENT,
            response.status_code
        )

    def test_remove_computer_system_task_error(self):
        """Tests remove ComputerSystem with task completed"""

        task = {'taskState': 'Error'}

        self.oneview_client.server_profiles.delete.return_value = task

        response = self.client.delete(
            "/redfish/v1/Systems/"
            "e7f93fa2-0cb4-11e8-9060-e839359bc36b")

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_all_computer_system_health_status(self, get_map_appliances,
                                               get_map_resources):
        server_hardware = copy.deepcopy(self.server_hardware)
        expected_cs = copy.deepcopy(self.computer_system_mockup)
        server_profile = copy.deepcopy(self.server_profile)
        server_profile["localStorage"]["sasLogicalJBODs"].pop(0)

        for oneview_status, redfish_status in \
                status_mapping.HEALTH_STATE.items():
            server_hardware["status"] = oneview_status
            expected_cs["Status"]["Health"] = redfish_status

            resource_id = "/rest/server-hardware-types/" \
                          "FE50A6FE-B1AC-4E42-8D40-B73CA8CC0CD2"
            get_map_resources.return_value = OrderedDict({
                resource_id: "10.0.0.1",
            })
            get_map_appliances.return_value = self.map_appliance
            self.oneview_client.server_profiles.get.return_value = \
                server_profile
            self.oneview_client.server_hardware.get.return_value = \
                server_hardware
            self.oneview_client.server_hardware_types.get.return_value = \
                self.server_hardware_types
            self.oneview_client.sas_logical_jbods.get_drives.return_value = \
                [self.drives[4]]
            self.oneview_client.labels.get_by_resource.return_value = \
                self.label_for_server_profile

            response = self.client.get(
                "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2"
            )

            result = json.loads(response.data.decode("utf-8"))

            self.assertEqual(status.HTTP_200_OK, response.status_code)
            self.assertEqual("application/json", response.mimetype)
            self.assertEqualMockup(expected_cs, result)

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_all_computer_system_states(self, get_map_appliances,
                                        get_map_resources):
        expected_cs = copy.deepcopy(self.computer_system_mockup)
        server_profile = copy.deepcopy(self.server_profile)
        server_profile["localStorage"]["sasLogicalJBODs"].pop(0)

        for oneview_status, redfish_status in status_mapping.\
                SERVER_PROFILE_STATE_TO_REDFISH_STATE.items():

            server_profile["state"] = oneview_status
            expected_cs["Status"]["State"] = redfish_status

            resource_id = "/rest/server-hardware-types/" \
                          "FE50A6FE-B1AC-4E42-8D40-B73CA8CC0CD2"
            get_map_resources.return_value = OrderedDict({
                resource_id: "10.0.0.1",
            })
            get_map_appliances.return_value = self.map_appliance
            self.oneview_client.server_profiles.get.return_value = \
                server_profile
            self.oneview_client.server_hardware.get.return_value = \
                self.server_hardware
            self.oneview_client.server_hardware_types.get.return_value = \
                self.server_hardware_types
            self.oneview_client.sas_logical_jbods.get_drives.return_value = \
                [self.drives[4]]
            self.oneview_client.labels.get_by_resource.return_value = \
                self.label_for_server_profile
            self.oneview_client.appliance_node_information.get_version.return_value = \
                self.appliance_info

            response = self.client.get(
                "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2"
            )

            result = json.loads(response.data.decode("utf-8"))

            self.assertEqual(status.HTTP_200_OK, response.status_code)
            self.assertEqual("application/json", response.mimetype)
            self.assertEqualMockup(expected_cs, result)

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_computer_system_sp_with_invalid_labels(self,
                                                        get_map_appliances,
                                                        get_map_resources):
        """Tests ComputerSystem with a known SP with invalid labels"""

        server_profile = copy.deepcopy(self.server_profile)
        server_profile["localStorage"]["sasLogicalJBODs"].pop(0)
        invalid_label = {
            "uri": "/rest/labels/2",
            "name": "invalid label abc123"
        }

        label_for_sp = copy.deepcopy(self.label_for_server_profile)
        label_for_sp["labels"].extend(invalid_label)

        server_profile_template = copy.deepcopy(self.server_profile_template)
        server_profile_template["uri"] = self.spt_uri

        # Create mock response
        resource_id = "/rest/server-hardware-types/" \
                      "FE50A6FE-B1AC-4E42-8D40-B73CA8CC0CD2"
        get_map_resources.return_value = OrderedDict({
            resource_id: "10.0.0.1",
        })
        get_map_appliances.return_value = self.map_appliance
        self.oneview_client.server_profiles.get.return_value = server_profile
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware
        self.oneview_client.server_hardware_types.get.return_value = \
            self.server_hardware_types
        self.oneview_client.sas_logical_jbods.get_drives.return_value = \
            [self.drives[4]]
        self.oneview_client.labels.get_by_resource.return_value = label_for_sp
        self.oneview_client.server_profile_templates.get.return_value = \
            server_profile_template
        self.oneview_client.appliance_node_information.get_version.return_value = \
            self.appliance_info

        # Get ComputerSystem
        response = self.client.get(
            "/redfish/v1/Systems/b425802b-a6a5-4941-8885-aab68dfa2ee2"
        )

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.computer_system_mockup, result)
        self.assertEqual(
            "{}{}".format("W/", self.server_profile["eTag"]),
            response.headers["ETag"])
        self.oneview_client.server_profiles.get.assert_called_with(
            "b425802b-a6a5-4941-8885-aab68dfa2ee2"
        )
        self.oneview_client.server_hardware.get.assert_called_with(
            "/rest/server-hardware/30303437-3034-4D32-3230-313130304752"
        )
        self.oneview_client.server_hardware_types.get.assert_called_with(
            "/rest/server-hardware-types/FE50A6FE-B1AC-4E42-8D40-B73CA8CC0CD2"
        )
        self.oneview_client.sas_logical_jbods.get_drives.assert_called_with(
            "/rest/sas-logical-jbods/9e83a03d-7a84-4f0d-a8d7-bd05a30c3175"
        )

        self.oneview_client.labels.get_by_resource.assert_called_with(
            "/rest/server-profiles/b425802b-a6a5-4941-8885-aab68dfa2ee2"
        )

        self.oneview_client.server_profile_templates.get.assert_called_with(
            "61c3a463-1355-4c68-a4e3-4f08c322af1b"
        )



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.storage_drive_composition_details import \
    StorageDriveCompositionDetails
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestStorageDriveCompositionDetails(BaseTest):
    """Tests for StorageDriveCompositionDetails class"""

    def setUp(self):
        """Tests preparation"""

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Drive.json'
        ) as f:
            self.drive = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/DriveEnclosure.json'
        ) as f:
            self.drive_enclosure = json.load(f)

    def test_serialize(self):
        with open(
            'oneview_redfish_toolkit/mockups/redfish'
            '/StorageDriveCompositionDetails.json'
        ) as f:
            expected_result = json.load(f)

        target = StorageDriveCompositionDetails(self.drive,
                                                self.drive_enclosure)
        result = json.loads(target.serialize())

        self.assertEqualMockup(expected_result, result)



# -*- coding: utf-8 -*-

# Copyright (2017) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import g
from flask import Response
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# own libs

from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishError
from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishResourceNotFoundError
from oneview_redfish_toolkit.api.network_port import \
    NetworkPort

network_port = Blueprint("network_port", __name__)


@network_port.route(
    "/redfish/v1/Chassis/<uuid>/NetworkAdapters/<device_id>"
    "/NetworkPorts/<port_id>",
    methods=["GET"])
def get_network_port(uuid, device_id, port_id):
    """Get the Redfish NetworkPort for a given UUID, device_id and port_id.

        Return NetworkPort Redfish JSON for a given hardware UUID, device_id
        and port_id.

        Parameters:
            uuid: the UUID of the server_hardware
            device_id: The id of the network device
            port_id: The id of the port

        Returns:
            JSON: Redfish json with NetworkPort

        Exceptions:
            When hardware is not found calls abort(404)
            When other errors occur calls abort(500)

    """
    try:
        # Initial validation of device_id
        device_id_validation = int(device_id)

        server_hardware = g.oneview_client.server_hardware.get(uuid)

        # Final validation of device_id
        if device_id_validation - 1 < 0 or (device_id_validation - 1) >= \
            len(server_hardware["portMap"]["deviceSlots"]):
            raise OneViewRedfishResourceNotFoundError(
                device_id, "Network adapter")

        np = NetworkPort(device_id, port_id, server_hardware)

        json_str = np.serialize()

        return Response(
            response=json_str,
            status=status.HTTP_200_OK,
            mimetype="application/json")
    except ValueError:
        # Failed to convert device_id to int
        logging.exception(
            "Failed to convert device id {} to integer.".format(device_id))
        abort(status.HTTP_404_NOT_FOUND, "Network adapter not found")
    except OneViewRedfishResourceNotFoundError as e:
        logging.exception(e.msg)
        abort(status.HTTP_404_NOT_FOUND, e.msg)
    except OneViewRedfishError as e:
        logging.exception(e.msg)
        abort(status.HTTP_404_NOT_FOUND, e.msg)
    except HPOneViewException as e:
        # In case of error log exception and abort
        logging.exception(e)
        if e.oneview_response['errorCode'] == "RESOURCE_NOT_FOUND":
            abort(status.HTTP_404_NOT_FOUND, "Server hardware not found")
        else:
            abort(status.HTTP_500_INTERNAL_SERVER_ERROR)
    except Exception as e:
        # In case of error log exception and abort
        logging.exception('Unexpected error: {}'.format(e))
        abort(status.HTTP_500_INTERNAL_SERVER_ERROR)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api import errors
from oneview_redfish_toolkit.api.redfish_error import RedfishError
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestRedfishError(BaseTest):
    """Tests for RedfishError class"""

    def test_class_instantiation(self):
        """Tests class instantiation"""

        try:
            redfish_error = RedfishError("GeneralError", "General Error")
        except Exception as e:
            self.fail("Failed to instantiate RedfishError. Error: ".format(e))
        self.assertIsInstance(redfish_error, RedfishError)

    def test_serialize(self):
        """Tests the serialize function result against known result"""

        redfish_error = RedfishError("GeneralError", "General Error")
        result = json.loads(redfish_error.serialize())

        with open(
            'oneview_redfish_toolkit/mockups/errors/'
            'RedfishErrorNoExtendedInfo.json'
        ) as f:
            redfish_error_mockup = json.load(f)
        self.assertEqualMockup(redfish_error_mockup, result)

    def test_add_extended_info_invalid_error_code(self):
        """Tests the add_extended_info invalid error code"""

        redfish_error = RedfishError("GeneralError", "General Error")

        try:
            redfish_error.add_extended_info(
                "InvalidCode",
                "General Message")
        except errors.OneViewRedfishResourceNotFoundError as e:
            self.assertEqual(
                e.msg,
                "message_id InvalidCode not found")

    def test_add_extended_info_invalid_message_args(self):
        """Tests the add_extended_info invalid message_args"""

        redfish_error = RedfishError("GeneralError", "General Error")

        try:
            redfish_error.add_extended_info(
                message_id="PropertyValueNotInList",
                message_args=["Only 1, need 2"])
        except errors.OneViewRedfishError as e:
            self.assertEqual(
                e.msg,
                'Message has 2 replacements to be made but 1 args where sent')

    def test_redfish_error_with_extended_info(self):
        """Tests the add_extended_info with two additional info"""

        with open(
            'oneview_redfish_toolkit/mockups/errors/'
            'RedfishErrorExtendedInfo.json'
        ) as f:
            redfish_error_mockup = json.load(f)

        try:
            redfish_error = RedfishError(
                "GeneralError",
                "A general error has occurred. See ExtendedInfo "
                "for more information.")
            redfish_error.add_extended_info(
                message_id="PropertyValueNotInList",
                message_args=["RED", "IndicatorLED"],
                related_properties=["#/IndicatorLED"])
            redfish_error.add_extended_info(
                message_id="PropertyNotWritable",
                message_args=["SKU"],
                related_properties=["#/SKU"])
        except errors.OneViewRedfishError as e:
            self.fail("Failled to add Extened info".format(e))

        result = json.loads(redfish_error.serialize())
        self.assertEqualMockup(redfish_error_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for redfish_json_validator.py
"""

import json
from unittest import mock

from oneview_redfish_toolkit.api.service_root import ServiceRoot
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestServiceRoot(BaseTest):
    """Tests for ServiceRoot class"""

    @mock.patch.object(config, 'get_authentication_mode')
    def test_when_auth_mode_is_session(self, config_mock):
        config_mock.return_value = "session"

        service_root = ServiceRoot('00000000-0000-0000-0000-000000000000')
        result = json.loads(service_root.serialize())

        with open(
            'oneview_redfish_toolkit/mockups/redfish/ServiceRoot.json'
        ) as f:
            service_root_mockup = json.load(f)

        self.assertEqualMockup(service_root_mockup, result)

    @mock.patch.object(config, 'get_authentication_mode')
    def test_when_auth_mode_is_conf(self, config_mock):
        config_mock.return_value = "conf"

        service_root = ServiceRoot('00000000-0000-0000-0000-000000000000')
        result = json.loads(service_root.serialize())

        with open(
                'oneview_redfish_toolkit/mockups/redfish/ServiceRoot.json'
        ) as f:
            service_root_mockup = json.load(f)
            service_root_mockup['Links']['Sessions'] = {}

        self.assertEqualMockup(service_root_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for computer_system_collection.py
"""

import json

from oneview_redfish_toolkit.api.computer_system_collection \
    import ComputerSystemCollection
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestComputerSystemCollection(BaseTest):
    """Tests for ComputerSystemCollection class"""

    def setUp(self):
        """Tests preparation"""

        # Loading ServerHardware list mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerHardwareListWithServerProfileApplied.json'
        ) as f:
            self.server_hardware_list = json.load(f)

        # Loading ComputerSystemCollection result mockup
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'ComputerSystemCollection.json'
        ) as f:
            self.computer_system_collection_mockup = json.load(f)

        # Loading ServerProfileTemplates result mockup
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerProfileTemplates.json'
        ) as f:
            self.server_profile_template_list = json.load(f)

    def test_serialize(self):
        # Tests the serialize function result against known result

        zone_ids = [
            "1f0ca9ef-7f81-45e3-9d64-341b46cf87e0-0000000000A66101",
            "1f0ca9ef-7f81-45e3-9d64-341b46cf87e0-0000000000A66102",
            "75871d70-789e-4cf9-8bc8-6f4d73193578"
        ]

        computer_system_collection = ComputerSystemCollection(
            self.server_hardware_list,
            self.server_profile_template_list,
            zone_ids
        )

        result = json.loads(computer_system_collection.serialize())

        self.assertEqualMockup(self.computer_system_collection_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging


def debug(logger_name, *texts):
    if logging.getLogger().isEnabledFor(logging.DEBUG):
        msg = ", ".join(texts)
        logger = logging.getLogger(logger_name)
        logger.debug(msg)
